package com.studioirregular.tatala.check;

import java.util.ArrayList;
import java.util.List;

import com.studioirregular.tatala.BaseActivity;
import com.studioirregular.tatala.Engine;
import com.studioirregular.tatala.Engine.EngineParams;
import com.studioirregular.tatala.component.AnimationComponent;
import com.studioirregular.tatala.component.GameComponent;
import com.studioirregular.tatala.component.MoveAnimation;
import com.studioirregular.tatala.component.RenderComponent;
import com.studioirregular.tatala.component.RotateAnimation;
import com.studioirregular.tatala.component.ScaleAnimation;
import com.studioirregular.tatala.entity.Entity;
import com.studioirregular.tatala.entity.Scene;
import com.studioirregular.tatala.opengl.TextureSystem;
import com.studioirregular.tatala.opengl.TextureSystem.CreateRegion;
import com.studioirregular.tatala.opengl.TextureSystem.CreateTexture;
import com.studioirregular.tatala.util.Color;

public class DemoEntity extends BaseActivity {

	@Override
	protected EngineParams obtainEngineParams() {
		return new Engine.EngineParams(720, 480);
	}

	@Override
	protected void onLoadResource() {
	    CreateRegion region = new CreateRegion("background", 0, 0, 720, 450);
	    List<CreateRegion> regionList = new ArrayList<CreateRegion>();
	    regionList.add(region);
	    TextureSystem.getInstance().add(new CreateTexture("texture_1", R.drawable.texture_1), regionList);
	}
	
	@Override
	protected Scene onBuildScene() {
		//Scene scene = new Scene("scene", 720, 480, new Color(0.8f, 0.8f, 0.8f, 1.0f));
	    Scene scene = new Scene("scene", 720, 480, "texture_1", "background");
		
		// 3 overlapped entities.
		Entity obj1 = new Entity("obj1", 128, 128, 256, 256);
		GameComponent render = new RenderComponent("obj1-render", new Color(0.0f, 1.0f, 0, 1.0f));
		obj1.add(render);
		
		Entity obj2 = new Entity("obj2", 64, 32, 196, 196);
		render = new RenderComponent("obj2-render", new Color(1.0f, 0.0f, 0.0f, 0.6f));
		obj2.add(render);
		
		Entity obj3 = new Entity("obj3", 224, 64, 128, 128);
		render = new RenderComponent("obj3-render", new Color(0.0f, 0.0f, 1.0f, 0.6f));
		obj3.add(render);
		
		obj1.add(obj2);
		obj1.add(obj3);
		
		scene.add(obj1);
		
		// moving entity
		Entity entity = new Entity(420, 32, 128, 128);
		entity.add(new RenderComponent(new Color(1.0f, 1.0f, 0.5f, 1.0f)));
		
		AnimationComponent anim = new MoveAnimation(3000, true, 420, 540, 32, 32);
		entity.add(anim);
		anim.start();
		
		scene.add(entity);
		
		// scaling entity
		entity = new Entity(480, 196, 128, 127);
		entity.add(new RenderComponent(new Color(0.5f, 1.0f, 1.0f, 1.0f)));
		
		anim = new ScaleAnimation(1500, true, 1.0f, 2.0f, 1.0f, 1.5f);
		entity.add(anim);
		anim.start();
		
		scene.add(entity);
		
		// rotating entity
		entity = new Entity(480, 300, 128, 128);
		entity.add(new RenderComponent(new Color(1.0f, 0.5f, 1.0f, 1.0f)));
		
		anim = new RotateAnimation(2400, true, 0, 360);
		entity.add(anim);
		anim.start();
		
		scene.add(entity);
		
		return scene;
	}
	
}
