package com.studioirregular.tatala.check;

import java.util.Timer;
import java.util.TimerTask;

import android.util.Log;

import com.studioirregular.tatala.BaseActivity;
import com.studioirregular.tatala.Engine;
import com.studioirregular.tatala.Engine.EngineParams;
import com.studioirregular.tatala.component.AnimationComponent;
import com.studioirregular.tatala.component.RenderComponent;
import com.studioirregular.tatala.component.RotateAnimation;
import com.studioirregular.tatala.component.ScaleAnimation;
import com.studioirregular.tatala.entity.Entity;
import com.studioirregular.tatala.entity.Scene;
import com.studioirregular.tatala.util.Color;
import com.studioirregular.tatala.util.ProfileRecorder;

public class DemoProfiling extends BaseActivity {

    private static final int ENTITY_COUNT = 128;
    
    private static final float SCENE_WIDTH = 720;
    private static final float SCENE_HEIGHT = 480;
    private static final float MIN_ENTITY_WIDTH = 32;
    private static final float MAX_ENTITY_WIDTH = 128;
    
    private static final String TAG = "demo-profiling";
    
    @Override
    protected EngineParams obtainEngineParams() {
        return new Engine.EngineParams(SCENE_WIDTH, SCENE_HEIGHT);
    }

    @Override
    protected void onLoadResource() {
    }

    @Override
    protected Scene onBuildScene() {
        Scene scene = new Scene(SCENE_WIDTH, SCENE_HEIGHT, new Color(0, 0, 0, 0));
        
        for (int i = 0; i < ENTITY_COUNT; i++) {
            final float x = (float)Math.random() * SCENE_WIDTH;
            final float y = (float)Math.random() * SCENE_HEIGHT;
            final float width = MIN_ENTITY_WIDTH + ((float)Math.random() * (MAX_ENTITY_WIDTH - MIN_ENTITY_WIDTH));
            
            final float r = (float)Math.random();
            final float g = (float)Math.random();
            final float b = (float)Math.random();
            final float alpha = 0.5f + (float)Math.random() * 0.5f;
            Entity entity = new Entity(x, y, width, width);
            entity.add(new RenderComponent(new Color(r, g, b, alpha)));
            
            final int duration = 1000 + (int)(Math.random() * 2000);
            AnimationComponent anim = new RotateAnimation(duration, true, 0, 360);
            anim.start();
            entity.add(anim);
            
            anim = new ScaleAnimation(duration, true, 1.0f, 1.5f, 1.0f, 1.5f);
            anim.start();
            entity.add(anim);
            
            scene.add(entity);
        }
        
        return scene;
    }
    
    @Override
    protected void onResume() {
        super.onResume();
        
        profileTimer = new Timer();
        profileTimer.scheduleAtFixedRate(new ProfileLogger(), 2000, 2000);
    }

    @Override
    protected void onPause() {
        super.onPause();
        
        profileTimer.cancel();
    }
    
    private Timer profileTimer;

    private class ProfileLogger extends TimerTask {

        @Override
        public void run() {
            final ProfileRecorder rec = ProfileRecorder.getInstance();
            
            Log.d(TAG, "=====================================================");
            
            final long avgFrameTime = rec.getAverageTime(ProfileRecorder.PROFILE_FRAME);
            final long avgDrawTime = rec.getAverageTime(ProfileRecorder.PROFILE_DRAW);
            final long avgSwapTime = rec.getAverageTime(ProfileRecorder.PROFILE_SWAP_BUFFERS);
            final long avgUpdateTime = rec.getAverageTime(ProfileRecorder.PROFILE_UPDATE);
            Log.d(TAG, "AVG frame:" + avgFrameTime + ", draw:" + avgDrawTime + ", swap:" + avgSwapTime + ", update:" + avgUpdateTime);
            
            final long maxFrameTime = rec.getMaxTime(ProfileRecorder.PROFILE_FRAME);
            final long maxDrawTime = rec.getMaxTime(ProfileRecorder.PROFILE_DRAW);
            final long maxSwapTime = rec.getMaxTime(ProfileRecorder.PROFILE_SWAP_BUFFERS);
            final long maxUpdateTime = rec.getMaxTime(ProfileRecorder.PROFILE_UPDATE);
            Log.d(TAG, "MAX frame:" + maxFrameTime + ", draw:" + maxDrawTime + ", swap:" + maxSwapTime + ", update:" + maxUpdateTime);
            
            final long minFrameTime = rec.getMinTime(ProfileRecorder.PROFILE_FRAME);
            final long minDrawTime = rec.getMinTime(ProfileRecorder.PROFILE_DRAW);
            final long minSwapTime = rec.getMinTime(ProfileRecorder.PROFILE_SWAP_BUFFERS);
            final long minUpdateTime = rec.getMinTime(ProfileRecorder.PROFILE_UPDATE);
            Log.d(TAG, "MIN frame:" + minFrameTime + ", draw:" + minDrawTime + ", swap:" + minSwapTime + ", update:" + minUpdateTime);
            
            rec.resetAll();
        }
        
    }

}
