package com.studioirregular.tatala;

import android.app.Activity;
import android.os.Bundle;
import android.view.MotionEvent;
import android.view.Window;
import android.view.WindowManager;

import com.studioirregular.tatala.entity.Scene;
import com.studioirregular.tatala.opengl.GameRenderer;
import com.studioirregular.tatala.opengl.MyGLSurfaceView;

public abstract class BaseActivity extends Activity {

	@Override
	protected void onCreate(Bundle savedInstanceState) {
		super.onCreate(savedInstanceState);
		
		final Engine.EngineParams params = obtainEngineParams();
		applyEngineParams(params);
		
		MyGLSurfaceView view = new MyGLSurfaceView(this);
		
		this.engine = new Engine(params, this);
		
		GameRenderer renderer = new GameRenderer(engine);
		view.setRenderer(renderer);
		
		setContentView(view);
		
		this.engine.start(view, renderer);
	}
	
	@Override
    protected void onResume() {
        super.onResume();
        
        this.onLoadResource();
        this.engine.setScene(onBuildScene());
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        
        if (this.engine != null) {
            this.engine.stop();
        }
    }
    
    @Override
    public boolean onTouchEvent(MotionEvent event) {
        InputSystem.getInstance().onTouch(event);
        return true;
    }

    protected abstract Engine.EngineParams obtainEngineParams();
	protected abstract Scene onBuildScene();
	protected abstract void onLoadResource();
	
	protected Engine engine;
	
	protected void applyEngineParams(Engine.EngineParams params) {
		if (params.isFullScreen()) {
			requestFullscreen();
		}
	}
	
	private void requestFullscreen() {
		final Window window = getWindow();
		window.addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
		window.clearFlags(WindowManager.LayoutParams.FLAG_FORCE_NOT_FULLSCREEN);
		window.requestFeature(Window.FEATURE_NO_TITLE);
	}
	
}
