package com.studioirregular.tatala.component;

import com.studioirregular.tatala.GameObject;
import com.studioirregular.tatala.entity.Entity;

public abstract class AnimationComponent extends GameComponent {

    public AnimationComponent(int durationInMilliSeconds, boolean loop) {
        this.duration = durationInMilliSeconds;
        this.loop = loop;
    }
    
    public void start() {
        this.elapsedTime = 0;
        this.started = true;
    }
    
    public void stop() {
        this.started = false;
    }
    
    @Override
    public void update(int timeDelta, GameObject parent) {
        super.update(timeDelta, parent);
        
        if (!this.started) {
            return;
        }
        
        this.elapsedTime = this.elapsedTime + timeDelta;
        
        if (this.duration < this.elapsedTime) {
            if (this.loop) {
                this.elapsedTime = this.elapsedTime % this.duration;
            } else {
                stop();
                return;
            }
        }
        
        doUpdate(this.elapsedTime, (Entity)parent);
    }
    
    protected abstract void doUpdate(int animTime, Entity parent);

    protected int duration;
    protected boolean loop;
    protected boolean started;
    protected int elapsedTime;
}
