package com.studioirregular.tatala.component;

import android.graphics.RectF;

import com.studioirregular.tatala.GameObject;
import com.studioirregular.tatala.InputSystem;
import com.studioirregular.tatala.InputSystem.TouchEvent;
import com.studioirregular.tatala.entity.Entity;

public class ButtonComponent extends GameComponent {

    public ButtonComponent(String id, float left, float top, float right, float bottom) {
        super(id);
        box = new RectF(left, top, right, bottom);
        buttonDown = false;
    }
    
    public void setPresentation(RenderComponent up, RenderComponent down) {
        this.up = up;
        this.down = down;
    }
    
    @Override
    public void update(int timeDelta, GameObject parent) {
        super.update(timeDelta, parent);
        
        final InputSystem input = InputSystem.getInstance();
        input.lock();
        
        TouchEvent touch = input.next();
        while (touch != null) {
            if ((TouchEvent.DOWN == touch.action) &&
                (box.contains(touch.x, touch.y))) {
                buttonDown = true;
                notifyStateChanged((Entity)parent);
                
                input.remove(touch);
            } else if ((TouchEvent.UP == touch.action) &&
                       (buttonDown)) {
                buttonDown = false;
                notifyStateChanged((Entity)parent);
                
                input.remove(touch);
            } else if (buttonDown && TouchEvent.MOVE == touch.action) {
                input.remove(touch);
            }
            
            touch = input.next();
        }
        
        input.unlock();
    }
    
    protected void notifyStateChanged(Entity parent) {
        if (buttonDown) {
            parent.remove(up);
            parent.add(down);
        } else {
            parent.remove(down);
            parent.add(up);
        }
    }
    
    protected RectF box;
    protected boolean buttonDown;
    
    protected RenderComponent down, up;

}
