package com.studioirregular.tatala.component;

import com.studioirregular.tatala.GameObject;
import com.studioirregular.tatala.InputSystem;
import com.studioirregular.tatala.InputSystem.TouchEvent;
import com.studioirregular.tatala.entity.Entity;

public class DraggableComponent extends GameComponent {

    public DraggableComponent(String id) {
        super(id);
        dragging = false;
    }
    
    public void setPresentation(RenderComponent normal, RenderComponent dragged) {
        this.normal = normal;
        this.dragged = dragged;
    }
    
    @Override
    public void update(int timeDelta, GameObject parent) {
        super.update(timeDelta, parent);
        
        final InputSystem input = InputSystem.getInstance();
        final Entity host = (Entity)parent;
        
        input.lock();
        
        TouchEvent touch = input.next();
        while (touch != null) {
            if ((TouchEvent.DOWN == touch.action) &&
                (host.box.contains(touch.x, touch.y))) {
                dragging = true;
                downX = touch.x;
                downY = touch.y;
                downParentX = host.box.left;
                downParentY = host.box.top;
                
                notifyStateChanged(host);
                
                input.remove(touch);
            } else if (dragging) {
                final float newX = downParentX + touch.x - downX;
                final float newY = downParentY + touch.y - downY;
                host.box.offsetTo(newX, newY);
                
                input.remove(touch);
                
                if (TouchEvent.UP == touch.action) {
                    dragging = false;
                    notifyStateChanged(host);
                }
            }
            
            touch = input.next();
        }
        
        input.unlock();
    }
    
    protected void notifyStateChanged(Entity parent) {
        if (dragging) {
            parent.remove(normal);
            parent.add(dragged);
        } else {
            parent.remove(dragged);
            parent.add(normal);
        }
    }
    
    private boolean dragging;
    private float downX, downY;
    private float downParentX, downParentY;
    
    private RenderComponent normal, dragged;

}
