package com.studioirregular.tatala.component;

import android.graphics.RectF;

import com.studioirregular.tatala.GameObject;
import com.studioirregular.tatala.entity.Entity;
import com.studioirregular.tatala.graphics.ColorDrawable;
import com.studioirregular.tatala.graphics.RenderSystem;
import com.studioirregular.tatala.graphics.RenderSystem.Drawable;
import com.studioirregular.tatala.graphics.TextureDrawable;
import com.studioirregular.tatala.opengl.Texture;
import com.studioirregular.tatala.opengl.TextureRegion;
import com.studioirregular.tatala.opengl.TextureSystem;
import com.studioirregular.tatala.util.Color;
import com.studioirregular.tatala.util.Debug;

public class RenderComponent extends GameComponent {

    public RenderComponent(Color color) {
        this("unknown", color, null, null);
    }
    
    public RenderComponent(String id, Color color) {
        this(id, color, null, null);
    }
    
    public RenderComponent(String textureName, String regionName) {
        this("unknown", null, textureName, regionName);
    }
    
    public RenderComponent(String id, Color color, String textureName, String regionName) {
        super(id);
        this.color = color;
        this.textureName = textureName;
        this.textureRegionName = regionName;
    }
    
    public void setColor(Color color) {
        this.color.set(color);
    }
    
    @Override
    public void update(int timeDelta, GameObject parent) {
        super.update(timeDelta, parent);
        
        final Entity entity = (Entity)parent;
        
        if (textureDrawable == null) {
            if (textureName != null && textureRegionName != null) {
                Texture texture = TextureSystem.getInstance().getTexture(textureName);
                TextureRegion region = TextureSystem.getInstance().getRegion(textureName, textureRegionName);
                if (texture != null && region != null) {
                    textureDrawable = new TextureDrawable(entity.box.width(), entity.box.height(), texture, region);
                } else {
                    Debug.e("RenderComponent::update cannot find texture region, texture:" + textureName + ",region:" + textureRegionName);
                }
            }
        }
        
        if (textureDrawable != null && textureDrawable.getTexture().load) {
            final RectF box = entity.box;
            textureDrawable.x = box.left;
            textureDrawable.y = box.top;
            textureDrawable.scaleX = entity.scaleX;
            textureDrawable.scaleY = entity.scaleY;
            textureDrawable.rotation = entity.rotation;
            
            RenderSystem.getInstance().scheduleDrawable(textureDrawable);
        }
        
        if (colorDrawable == null && color != null) {
            colorDrawable = setupDrawable((Entity)parent);
        }
        
        if (colorDrawable != null) {
            colorDrawable.x = entity.box.left;
            colorDrawable.y = entity.box.top;
            colorDrawable.scaleX = entity.scaleX;
            colorDrawable.scaleY = entity.scaleY;
            colorDrawable.rotation = entity.rotation;
            
            RenderSystem.getInstance().scheduleDrawable(colorDrawable);
        }
    }
    
    protected Drawable setupDrawable(Entity parent) {
        final RectF box = parent.box;
        return new ColorDrawable(box.width(), box.height(), color);
    }
    
    protected Color color;
    protected Drawable colorDrawable;
    protected String textureName;
    protected String textureRegionName;
    protected TextureDrawable textureDrawable;

}
