package com.studioirregular.tatala.entity;

import javax.microedition.khronos.opengles.GL10;

import android.graphics.RectF;

/**
 * Game tree node, {@link GameObject} container. 
 * Also stores data shared among {@link GameComponent}.
 * 
 * Entity is not thread safe, it's critical to synchronize calls to 
 * {@link #onUpdate(long)} and {@link #onDraw(GL10)}, in that order. 
 * 
 * @author Michael Wang
 *
 */
public class Entity extends ObjectManager {

    public Entity(float x, float y, float width, float height) {
        this("unknown", x, y, width, height);
    }
    
	public Entity(String id, float x, float y, float width, float height) {
	    super(id);
	    
	    zIndex = 0;
		box = new RectF(x, y, x + width, y + height);
		scaleX = 1.0f;
		scaleY = 1.0f;
		rotation = 0;
	}
	
	public float getZIndex() {
		return zIndex;
	}
	
	/**
	 * Change z order.
	 * TODO: notify parent to sort its children.
	 * 
	 * @param zIndex z order value, >= 0.
	 */
	public void setZIndex(int zIndex) {
		this.zIndex = zIndex;
	}
	
	protected int zIndex;  // unused yet..
	
	// info shared with components
	public RectF box;
	public float scaleX;
	public float scaleY;
	public float rotation;
	
}
