package com.studioirregular.tatala.entity;

import java.util.ArrayList;
import java.util.List;

import com.studioirregular.tatala.GameObject;

/**
 * Manage game objects: add, remove, update.
 * 
 * Subclass should call {@link #update(long, GameObject)} if override it.
 * 
 * @author Michael Wang
 *
 */
public class ObjectManager extends GameObject {

    public ObjectManager() {
        this("unknown");
    }
    
    public ObjectManager(String id) {
        super(id);
        objects = new ArrayList<GameObject>();
        pendingAdd = new ArrayList<GameObject>();
        pendingRemove = new ArrayList<GameObject>();
    }
    
    public void add(GameObject o) {
        pendingAdd.add(o);
    }
    
    public void remove(GameObject o) {
        pendingRemove.add(o);
    }
    
    @Override
    public void update(int timeDelta, GameObject parent) {
//        Debug.d(this.getClass().getSimpleName() + " id:" + id + " >> update parent:" + (parent != null ? parent.id : "null"));
        commitUpdate();
        
        final int count = objects.size();
        for(int i = (count - 1); i >= 0; i--) {
            final GameObject obj = objects.get(i);
            obj.update(timeDelta, this);
        }
    }
    
    protected void commitUpdate() {
        int count = pendingAdd.size();
        for (int i = 0; i < count; i++) {
            objects.add(pendingAdd.get(i));
        }
        pendingAdd.clear();
        
        count = pendingRemove.size();
        for (int i = 0; i < count; i++) {
            objects.remove(pendingRemove.get(i));
        }
        pendingRemove.clear();
    }
    
    protected List<GameObject> objects;
    private List<GameObject> pendingAdd;
    private List<GameObject> pendingRemove;
}
