package com.studioirregular.tatala.graphics;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.nio.FloatBuffer;

import javax.microedition.khronos.opengles.GL10;

import com.studioirregular.tatala.graphics.RenderSystem.Drawable;
import com.studioirregular.tatala.opengl.Texture;
import com.studioirregular.tatala.opengl.TextureRegion;

public class TextureDrawable extends Drawable {

    public TextureDrawable(float width, float height, Texture texture, TextureRegion region) {
        super(width, height);
        
        this.texture = texture;
        textureCoord = buildTextureCoordinate(texture, region);
    }
    
    public Texture getTexture() {
        return texture;
    }
    
    @Override
    protected void draw(GL10 gl) {
        if (texture.load) {
            gl.glEnableClientState(GL10.GL_VERTEX_ARRAY);
            gl.glVertexPointer(3, GL10.GL_FLOAT, 0, vertex);
            
            gl.glEnableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
            gl.glTexCoordPointer(2, GL10.GL_FLOAT, 0, textureCoord);
            gl.glBindTexture(GL10.GL_TEXTURE_2D, texture.hwId);
            
            gl.glDrawArrays(GL10.GL_TRIANGLE_STRIP, 0, 4);
            
            gl.glDisableClientState(GL10.GL_TEXTURE_COORD_ARRAY);
            gl.glDisableClientState(GL10.GL_VERTEX_ARRAY);
        }
    }
    
    private FloatBuffer buildTextureCoordinate(Texture texture, TextureRegion region) {
        FloatBuffer result = null;
        
        ByteBuffer tbb = ByteBuffer.allocateDirect(2 * 4 * 4);
        tbb.order(ByteOrder.nativeOrder());
        result = tbb.asFloatBuffer();
        
        final float left = (region.x + 0.5f) / texture.width;
        final float top = (region.y + 0.5f) / texture.height;
        final float right = (region.x + region.width - 0.5f) / texture.width;
        final float bottom = (region.y + region.height - 0.5f) / texture.height;
        float[] texCoord = { 
                left, bottom, 
                right, bottom, 
                left, top, 
                right, top
            };
        result.put(texCoord);
        result.position(0);
        
        return result;
    }
    
    private Texture texture;
    private FloatBuffer textureCoord;

}
