package com.studioirregular.tatala.opengl;

import java.util.concurrent.atomic.AtomicBoolean;

import javax.microedition.khronos.egl.EGLConfig;
import javax.microedition.khronos.opengles.GL10;

import android.graphics.RectF;
import android.opengl.GLU;

import com.studioirregular.tatala.Engine;
import com.studioirregular.tatala.graphics.RenderSystem;
import com.studioirregular.tatala.opengl.MyGLSurfaceView.MyRenderer;
import com.studioirregular.tatala.util.Debug;

public class GameRenderer implements MyRenderer {

	public GameRenderer(Engine engine) {
		this.engine = engine;
		drawLock = new Object();
		drawQueueUpdated = new AtomicBoolean(false);
	}
	
	@Override
	public void onSurfaceCreated(GL10 gl, EGLConfig config) {
	    gl.glEnable(GL10.GL_TEXTURE_2D);
	}
	
	@Override
	public void onSurfaceChanged(GL10 gl, int surfaceWidth, int surfaceHeight) {
		Debug.d("onSurfaceChanged w:" + surfaceWidth + ",h:" + surfaceHeight);
		final Engine.EngineParams options = engine.getParams();
		
		final float gameWidth = options.getWorldWidth();
		final float gameHeight = options.getWorldHeight();
		Debug.d("engine option w:" + gameWidth + ",h:" + gameHeight);
		
		// setup viewport
		final float scaleX = (float)surfaceWidth / gameWidth;
		final float scaleY = (float)surfaceHeight / gameHeight;
		final float scale = scaleX > scaleY ? scaleY : scaleX;
		Debug.d("scaleX:" + scaleX + ",scaleY:" + scaleY + ",scale:" + scale);
		
		int viewportWidth = 0;
		int viewportHeight = 0;
		if (scaleX > scaleY) {
			viewportWidth = (int)(gameWidth * scale);
			viewportHeight = surfaceHeight;	// avoid float multiplication (gameHeight * scale)
		} else {
			viewportWidth = surfaceWidth;	// avoid float multiplication (gameWidth * scale)
			viewportHeight = (int)(gameHeight * scale);
		}
		
		final int viewportX = (surfaceWidth - viewportWidth) / 2;
		final int viewportY = (surfaceHeight - viewportHeight) / 2;
		Debug.d("viewport x:" + viewportX + ",y:" + viewportY + ",w:" + viewportWidth + ",h:" + viewportHeight);
		
		gl.glViewport(viewportX, viewportY, viewportWidth, viewportHeight);
		
		// setup projection
		gl.glMatrixMode(GL10.GL_PROJECTION);
		gl.glLoadIdentity();
		GLU.gluOrtho2D(gl, 0, gameWidth, gameHeight, 0);
		
		viewportDimension.set(viewportX, viewportY, viewportX + viewportWidth, viewportY + viewportHeight);
	}
	
    @Override
    public void onDrawFrame(GL10 gl) {
        synchronized (drawLock) {
            if (!drawQueueUpdated.get()) {
                while (!drawQueueUpdated.get()) {
                    try {
                        drawLock.wait();
                    } catch (InterruptedException e) {

                    }
                }
            }
            drawQueueUpdated.set(false);
        }

        synchronized (this) {
            if (engine != null) {
                engine.onGLDraw(gl);
            }
            
            RenderSystem.getInstance().drawComplete();
        }
    }
	
	public void updateComplete() {
	    synchronized (drawLock) {
	        drawQueueUpdated.set(true);
	        drawLock.notifyAll();
	    }
	}
	
	public RectF getViewportDimension() {
	    return viewportDimension;
	}
	
	private Engine engine;
	private Object drawLock;
	private AtomicBoolean drawQueueUpdated;
	
	private RectF viewportDimension = new RectF();
}
