package com.studioirregular.tatala.opengl;

import java.nio.IntBuffer;

import javax.microedition.khronos.opengles.GL10;

import android.content.Context;
import android.graphics.Bitmap;
import android.graphics.BitmapFactory;
import android.opengl.GLUtils;

import com.studioirregular.tatala.util.Debug;

public class Texture {

    public static final int INVALID_RES_ID = -1;
    public static final int INVALID_HW_ID = -1;
    
    public String name;
    public int resId;
    public int hwId;
    public int width;
    public int height;
    public boolean load;
    
    public Texture(Context context, String name, int resId) {
        this.name = name;
        this.resId = resId;
        this.hwId = INVALID_HW_ID;
        this.width = 0;
        this.height = 0;
        updateDimension(context, resId);
        load = false;
    }
    
    public boolean load(Context context, GL10 gl) {
        Debug.d("Texture::load resId:" + resId);
        if (load || resId < 0 || context == null || gl == null) {
            return false;
        }
        
        Bitmap bitmap = null;
        try {
            bitmap = BitmapFactory.decodeResource(context.getResources(), resId);
        } catch (Exception e) {
            Debug.e("Texture::load BitmapFactory.decodeResource exception:" + e);
            return false;
        }
        
        int[] id = new int[1];
        gl.glGenTextures(1, id, 0);
        
        int errCode = gl.glGetError();
        if (errCode != GL10.GL_NO_ERROR) {
            Debug.e("Texture::load glGenTextures error:" + errCode);
            return false;
        }
        
        this.hwId = id[0];
        gl.glBindTexture(GL10.GL_TEXTURE_2D, hwId);
        
        errCode = gl.glGetError();
        if (errCode != GL10.GL_NO_ERROR) {
            Debug.e("Texture::load glBindTexture error:" + errCode);
            return false;
        }
        
        gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MIN_FILTER, GL10.GL_NEAREST);
        gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_MAG_FILTER, GL10.GL_LINEAR);
        
        gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_S, GL10.GL_CLAMP_TO_EDGE);
        gl.glTexParameterf(GL10.GL_TEXTURE_2D, GL10.GL_TEXTURE_WRAP_T, GL10.GL_CLAMP_TO_EDGE);
        
        errCode = gl.glGetError();
        if (errCode != GL10.GL_NO_ERROR) {
            Debug.e("Texture::load setup texture params error:" + errCode);
        }
        
        GLUtils.texImage2D(GL10.GL_TEXTURE_2D, 0, bitmap, 0);
        
        errCode = gl.glGetError();
        if (errCode != GL10.GL_NO_ERROR) {
            Debug.e("Texture::load GLUtils.texImage2D error:" + errCode);
            
            gl.glDeleteTextures(1, IntBuffer.wrap(id));
            return false;
        }
        
        bitmap.recycle();
        load = true;
        return true;
    }
    
    private void updateDimension(Context context, int resId) {
        BitmapFactory.Options opts = new BitmapFactory.Options();
        opts.inJustDecodeBounds = true;
        BitmapFactory.decodeResource(context.getResources(), resId, opts);
        
        this.width = opts.outWidth;
        this.height = opts.outHeight;
    }
}
